@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -18,8 +18,10 @@
 ## @deftypefn {Function File} {@var{Y} =} atan2rev1 (@var{B}, @var{C}, @var{Y})
 ## @deftypefnx {Function File} {@var{Y} =} atan2rev1 (@var{B}, @var{C})
 ## 
-## Compute the reverse atan2 function with
-## @code{atan2 (@var{Y}, @var{B}) = @var{C}}.
+## Compute the reverse atan2 function for the first parameter.
+##
+## That is, an enclosure of all @code{y ∈ @var{Y}} where
+## @code{atan2 (y, b) ∈ @var{C}} for any @code{b ∈ @var{B}}.
 ##
 ## Accuracy: The result is a valid enclosure.
 ##
@@ -161,3 +163,6 @@ else
     result = result | (y & infsup (l, u));
 endif
 endfunction
+
+%!test "from the documentation string";
+%! assert (atan2rev1 (infsup (1, 2), infsup ("pi") / 4) == "[0x1.FFFFFFFFFFFFFp-1, 0x1.0000000000001p1]");
