@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -20,6 +20,9 @@
 ## 
 ## Compute the reverse hyperbolic cosine function.
 ##
+## That is, an enclosure of all @code{x ∈ @var{X}} where
+## @code{cosh (x) ∈ @var{C}}.
+##
 ## Accuracy: The result is a tight enclosure.
 ##
 ## @example
@@ -58,3 +61,6 @@ n = - p;
 result = (p & x) | (n & x);
 
 endfunction
+
+%!test "from the documentation string";
+%! assert (coshrev (infsup (-2, 1)) == 0);
