@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -117,6 +117,9 @@ if (nargin == 1)
                     case {"empty", ""}
                         l {i} = inf;
                         u {i} = -inf;
+                    case "nai"
+                        error ("interval:NaI", ...
+                               "Interval literal [NaI] occured")
                     otherwise
                         boundaries = strtrim (strsplit (nobrackets, ","));
                         switch (numel (boundaries))
@@ -126,7 +129,8 @@ if (nargin == 1)
                                 l {i} = boundaries {1};
                                 u {i} = boundaries {2};
                             otherwise
-                                error ("interval literal is not in inf-sup form")
+                                error ("interval:InvalidOperand", ...
+                                    "interval literal is not in inf-sup form")
                         endswitch
                 endswitch
             elseif (ischar (l {i}) && strfind (c {i}, "?"))
@@ -174,28 +178,31 @@ if (ischar (u))
 endif
 
 if (not (size_equal (l, u)))
-    error ("size of upper and lower bounds must match")
+    error ("interval:InvalidOperand", ...
+           "infsup: size of upper and lower bounds must match")
 endif
 
 ## check parameters and conversion to double precision
 isexact = true ();
+possiblyundefined = false (size (l));
 x.inf = zeros (size (l));
 x.sup = zeros (size (u));
 input.inf = l;
 input.sup = u;
 for [boundaries, key] = input
     if (isfloat (boundaries))
-        if (any (any (isnan (boundaries))))
-            error (["illegal " key " boundary: NaN not allowed"]);
-        endif
         ## Simple case: the boundaries already are a binary floating point
         ## number in single or double precision.
         x.(key) = double (boundaries);
+        possiblyundefined (:) = false ();
         continue
     endif
     for i = 1 : numel (boundaries)
         if (iscell (boundaries))
             boundary = boundaries {i};
+            if (ischar (boundary))
+                boundary = lower (boundary);
+            endif
         else
             boundary = boundaries (i);
         endif
@@ -206,25 +213,32 @@ for [boundaries, key] = input
                     x.inf (i) = -inf;
                 case "sup"
                     x.sup (i) = inf;
+                    possiblyundefined (i) = false ();
             endswitch
         elseif (isnumeric (boundary))
             if (not (isreal (boundary)))
-                error (["illegal " key " boundary: must not be complex"]);
+                error ("interval:InvalidOperand", ...
+                       ["illegal " key " boundary: must not be complex"]);
             endif
             if (isfloat (boundary))
-                if (isnan (boundary))
-                    error (["illegal " key " boundary: NaN not allowed"]);
-                endif
                 ## Simple case: the boundary already is a binary floating point
                 ## number in single or double precision.
                 x.(key) (i) = double (boundary);
+                possiblyundefined (i) = false ();
                 continue
             endif
             
             ## Integer or logical, try to approximate in double precision
             x.(key) (i) = double (boundary);
             isdouble = x.(key) (i) == boundary;
-            isexact = and (isexact, isdouble);
+            if (not (isdouble))
+                isexact = false;
+                if (key == "inf")
+                    possiblyundefined (i) = true ();
+                endif
+            else
+                possiblyundefined (i) = false ();
+            end
             
             ## Check rounding direction of the approximation
             ## Mixed mode comparison works as intended
@@ -246,7 +260,8 @@ for [boundaries, key] = input
                 endswitch
             endif
         elseif (not (ischar (boundary)))
-            error (["illegal " key " boundary: must be numeric or string"]);
+            error ("interval:InvalidOperand", ...
+                   ["illegal " key " boundary: must be numeric or string"]);
         elseif (strfind (boundary, "0x"))
             ## Hexadecimal floating point number
             switch key
@@ -257,20 +272,38 @@ for [boundaries, key] = input
             endswitch
             
             [x.(key)(i), isdouble] = hex2double (boundary, direction);
-            isexact = and (isexact, isdouble);
+            if (not (isdouble))
+                isexact = false;
+                if (key == "inf")
+                    possiblyundefined (i) = true ();
+                endif
+            else
+                possiblyundefined (i) = false ();
+            end
         else
             ## parse string
+            if (key == "sup" && possiblyundefined (i) && iscell (l))
+                inf_boundary = l {i};
+                ## infsup ("x", "x") or infsup ("x") is not possibly undefined
+                if (strcmp (boundary, inf_boundary))
+                    possiblyundefined (i) = false;
+                endif
+            endif
+            
             switch (boundary)
                 case {"-inf", "-infinity"}
                     x.(key) (i) = -inf;
+                    possiblyundefined (i) = false ();
                 case {"inf", "+inf", "infinity", "+infinity"}
                     x.(key) (i) = inf;
+                    possiblyundefined (i) = false ();
                 case "e"
                     isexact = false ();
                     switch key
                         case "inf"
                             x.inf (i) = 0x56FC2A2 * pow2 (-25) ...
                                       + 0x628AED2 * pow2 (-52);
+                            possiblyundefined (i) = true ();
                         case "sup"
                             x.sup (i) = 0x56FC2A2 * pow2 (-25) ...
                                       + 0x628AED4 * pow2 (-52);
@@ -281,6 +314,7 @@ for [boundaries, key] = input
                         case "inf"
                             x.inf (i) = 0x6487ED5 * pow2 (-25) ...
                                       + 0x442D180 * pow2 (-55);
+                            possiblyundefined (i) = true ();
                         case "sup"
                             x.sup (i) = 0x6487ED5 * pow2 (-25) ...
                                       + 0x442D190 * pow2 (-55);
@@ -306,7 +340,8 @@ for [boundaries, key] = input
                         ## Special case: rational form
                         boundary = strsplit (boundary, "/");
                         if (length (boundary) ~= 2)
-                            error (["illegal " key " boundary: ", ...
+                            error ("interval:InvalidOperand", ...
+                                   ["illegal " key " boundary: ", ...
                                     "rational form must have single slash"]);
                         endif
                         [decimal, remainder] = decimaldivide (...
@@ -317,6 +352,9 @@ for [boundaries, key] = input
                             ## value
                             decimal.m (19, 1) = 1;
                             isexact = false ();
+                            if (key == "inf")
+                                possiblyundefined (i) = true ();
+                            endif
                         endif
                         ## Write result back into boundary for conversion to
                         ## double
@@ -366,6 +404,7 @@ for [boundaries, key] = input
                                 else # realmax ... inf
                                     x.inf (i) = realmax ();
                                 endif
+                                possiblyundefined (i) = true ();
                             case "sup"
                                 if (decimal.s) # -inf ... -realmax
                                     x.sup (i) = -realmax ();
@@ -383,7 +422,14 @@ for [boundaries, key] = input
                     ## Check approximation value
                     comparison = decimalcompare (double2decimal (binary), ...
                                                  decimal);
-                    isexact = and (isexact, comparison == 0);
+                    if (comparison ~= 0)
+                        isexact = false ();
+                        if (key == "inf")
+                            possiblyundefined (i) = true ();
+                        endif
+                    else
+                        possiblyundefined (i) = false ();
+                    endif
                     if (comparison == 0 || ... # approximation is exact
                         (comparison < 0 && key == "inf") || ... # lower bound
                         (comparison > 0 && key == "sup")) # upper bound
@@ -406,8 +452,11 @@ for [boundaries, key] = input
     endfor
 endfor
 
-assert (not (any (any (isnan (x.inf)))));
-assert (not (any (any (isnan (x.sup)))));
+## NaNs produce empty intervals
+nanvalue = isnan (x.inf) | isnan (x.sup);
+x.inf (nanvalue) = inf;
+x.sup (nanvalue) = -inf;
+possiblyundefined (nanvalue) = false;
 
 ## normalize boundaries:
 ## representation of the set containing only zero is always [-0,+0]
@@ -415,14 +464,26 @@ x.inf (x.inf == 0) = -0;
 x.sup (x.sup == 0) = +0;
 
 ## check for illegal boundaries [inf,inf] and [-inf,-inf]
-if (find (not (isfinite (x.inf (x.inf == x.sup))), 1))
-    error ("illegal interval boundaries: infimum = supremum = +/- infinity");
+if (any (any (not (isfinite (x.inf (x.inf == x.sup))), 1)))
+    error ("interval:InvalidOperand", ...
+           "illegal interval boundaries: infimum = supremum = +/- infinity");
 endif
 
 ## check boundary order
-if (find (isfinite (x.inf (x.inf > x.sup)), 1) || ...
-    find (isfinite (x.sup (x.inf > x.sup)), 1))
-    error ("illegal interval boundaries: infimum greater than supremum");
+if (any (any (isfinite (x.inf (x.inf > x.sup)))) || ...
+    any (any (isfinite (x.sup (x.inf > x.sup)))))
+    error ("interval:InvalidOperand", ...
+           "illegal interval boundaries: infimum greater than supremum");
+endif
+
+## check for possibly wrong boundary order
+if (any (any (possiblyundefined)) && any (any (max (-realmax, ...
+        mpfr_function_d ('plus',...
+                         +inf, ...
+                         x.inf (possiblyundefined), ...
+                         pow2 (-1074))) == x.sup (possiblyundefined))))
+    warning ("interval:PossiblyUndefined", ...
+             "infimum may be greater than supremum");
 endif
 
 x = class (x, "infsup");
@@ -433,11 +494,19 @@ endfunction
 %!  assert (sup (infsup (0)), 0);
 %!  assert (inf (infsup (2, 3)), 2);
 %!  assert (sup (infsup (2, 3)), 3);
+%!  assert (inf (infsup (nan)), +inf);
+%!  assert (sup (infsup (nan)), -inf);
+%!  assert (inf (infsup (nan, 2)), +inf);
+%!  assert (sup (infsup (nan, 2)), -inf);
+%!  assert (inf (infsup (3, nan)), +inf);
+%!  assert (sup (infsup (3, nan)), -inf);
 %!test "double matrix";
 %!  assert (inf (infsup (magic (4))), magic (4));
 %!  assert (sup (infsup (magic (4))), magic (4));
 %!  assert (inf (infsup (magic (3), magic (3) + 1)), magic (3));
 %!  assert (sup (infsup (magic (3), magic (3) + 1)), magic (3) + 1);
+%!  assert (inf (infsup (nan (3))), +inf (3));
+%!  assert (sup (infsup (nan (3))), -inf (3));
 %!test "decimal boundaries";
 %!  assert (inf (infsup ("0.1")), 0.1 - eps / 16);
 %!  assert (sup (infsup ("0.1")), 0.1);
@@ -510,5 +579,8 @@ endfunction
 %!  assert (sup (infsup ({"0.1", 42; "e", "3.2/8"}, {"0xffp2", "42e1"; "pi", 2})), [1020, 420; pi + 2 * eps, 2]);
 %!  assert (inf (infsup ({"[2, 3]", "3/4", "[Entire]", "42?3", 1, "0xf"})), [2, 0.75, -inf, 39, 1, 15]);
 %!  assert (sup (infsup ({"[2, 3]", "3/4", "[Entire]", "42?3", 1, "0xf"})), [3, 0.75, +inf, 45, 1, 15]);
+%!error infsup ("[nai]");
+%!error infsup ({42, "[nai]"});
 %!error infsup (3, 2);
 %!error infsup ("Ausgeschnitzel");
+%!warning infsup ("1.000000000000000000002", "1.000000000000000000001");
