@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -62,11 +62,13 @@ if (isscalar (x))
 endif
 
 if (not (isreal (y)) || fix (y) ~= y)
-    error ("mpower: only integral powers can be computed");
+    error ("interval:InvalidOperand", ...
+           "mpower: only integral powers can be computed");
 endif
 
 if (size (x, 1) ~= size (x, 2))
-    error ("mpower: must be square matrix");
+    error ("interval:InvalidOperand", ...
+           "mpower: must be square matrix");
 endif
 
 ## Implements log-time algorithm A.1 in
@@ -100,3 +102,6 @@ while (y ~= 0)
 endwhile
 
 endfunction
+
+%!test "from the documentation string";
+%! assert (infsup (magic (3)) ^ 2 == infsup (magic (3) ^ 2));
