@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 ##
 ## @example
 ## @group
-## infsup (-5, 6) ^ infsup (2, 3)
+## infsup (-5, 6) .^ infsup (2, 3)
 ##   @result{} [-125, +216]
 ## @end group
 ## @end example
@@ -284,3 +284,6 @@ else
     endif
 endif
 endfunction
+
+%!test "from the documentation string";
+%! assert (infsup (-5, 6) .^ infsup (2, 3) == infsup (-125, 216));
