@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -61,8 +61,8 @@
 ## magnitude (> 2^53) can also be affected from precision loss.
 ##
 ## Non-standard behavior: This class constructor is not described by IEEE 1788,
-## however it implements the IEEE 1788 functions newDec, numsToInterval, and
-## textToInterval.
+## however it implements the IEEE 1788 functions newDec, setDec, 
+## numsToInterval, and textToInterval.
 ## 
 ## @example
 ## @group
@@ -97,7 +97,8 @@ try
     if (nargin >= 1 && ...
         iscellstr (varargin {end}) && ...
         not (isempty (varargin {end})) && ...
-        max (strcmpi (varargin {end} {1}, {"com", "dac", "def", "trv"})))
+        max (strcmpi (varargin {end} {1}, ...
+                      {"com", "dac", "def", "trv", "ill"})))
         ## The decoration information has been passed as the last parameter
         dec = varargin {end};
         switch nargin
@@ -113,7 +114,8 @@ try
             case 3
                 [bare, isexact] = infsup (varargin {1}, varargin {2});
             otherwise
-                error ("too many arguments")
+                print_usage ();
+                return
         endswitch
     elseif (nargin == 1 && iscell (varargin {1}))
         ## Parse possibly decorated interval literals
@@ -126,7 +128,8 @@ try
                     varargin {1} {i} = literal {1};
                     dec {i} = literal {2};
                 elseif (length (literal) > 2)
-                       error ("illegal decorated interval literal")
+                    error ("interval:InvalidOperand", ...
+                           "illegal decorated interval literal")
                 endif
             endif
         endfor
@@ -150,7 +153,7 @@ try
             case 2
                 [bare, isexact] = infsup (varargin {1}, varargin {2});
             otherwise
-                error ("too many arguments")
+                error ("interval:InvalidOperand", "too many arguments")
         endswitch
     endif
     
@@ -166,8 +169,8 @@ try
         dec (:) = {decvalue};
     endif
     
-    if (not (min (size (dec) == size (bare))))
-        error ("decoration size mismatch")
+    if (not (all (size (dec) == size (bare))))
+        error ("interval:InvalidOperand", "decoration size mismatch")
     endif
     
     ## Add missing decoration
@@ -177,21 +180,33 @@ try
     dec (missingdecoration & iscommoninterval (bare)) = "com";
     
     ## Check decoration
-    if (max (max (isempty (bare) & not (strcmp (dec, "trv")))))
-        error ("illegal decoration for empty interval")
+    empty_not_trv = isempty (bare) & not (strcmp (dec, "trv"));
+    if (any (any (empty_not_trv)))
+        empty_not_trv = isempty (bare) & (strcmp (dec, "com") ...
+                                        | strcmp (dec, "dac") ...
+                                        | strcmp (dec, "def"));
+        isexact = false ();
+        dec (empty_not_trv) = "trv";
     endif
-    if (max (max (not (iscommoninterval (bare)) & strcmp (dec, "com"))))
-        error ("illegal decoration for uncommon interval")
+    uncommon_com = not (iscommoninterval (bare)) & strcmp (dec, "com");
+    if (any (any (uncommon_com)))
+        isexact = false ();
+        dec (uncommon_com) = "dac";
     endif
-    if (not (min (min ( ...
+    if (not (all (all ( ...
             strcmp (dec, "com") | ...
             strcmp (dec, "dac") | ...
             strcmp (dec, "def") | ...
             strcmp (dec, "trv")))))
-        error ("illegal decoration");
+        error ("interval:InvalidOperand", "illegal decoration");
     endif
 catch
-    warning (lasterror.message)
+    if (not (strcmp ("interval:NaI", lasterror.identifier)))
+        ## The bare inverval:NaI error can only occur, if the interval literal
+        ## [NaI] is observed. In that particular case, we must not issue a
+        ## warning.
+        warning ("interval:NaI", lasterror.message);
+    endif
     ## NaI representation is unique.
     bare = infsup ();
     dec = {"ill"};
@@ -210,10 +225,19 @@ superiorto ("infsup");
 endfunction
 
 %!# [NaI]s
+%!  assert (isnai (infsupdec ("[nai]"))); # quiet [NaI]
 %!warning assert (isnai (infsupdec (3, 2)));
 %!warning assert (isnai (infsupdec ("Flugeldufel")));
-%!warning assert (isnai (infsupdec (-inf, inf, "com")));
-%!warning assert (isnai (infsupdec (inf, -inf, "def")));
+%!test "decoration adjustments";
+%!  assert (inf (infsupdec (42, inf, "com")), 42);
+%!  assert (sup (infsupdec (42, inf, "com")), inf);
+%!  assert (strcmp (decorationpart (infsupdec (42, inf, "com")), "dac"));
+%!  assert (inf (infsupdec (-inf, inf, "com")), -inf);
+%!  assert (sup (infsupdec (-inf, inf, "com")), inf);
+%!  assert (strcmp (decorationpart (infsupdec (-inf, inf, "com")), "dac"));
+%!  assert (inf (infsupdec (inf, -inf, "def")), inf);
+%!  assert (sup (infsupdec (inf, -inf, "def")), -inf);
+%!  assert (strcmp (decorationpart (infsupdec (inf, -inf, "def")), "trv"));
 %!test "decorated interval literal";
 %!  assert (inf (infsupdec ("[2, 3]_def")), 2);
 %!  assert (sup (infsupdec ("[2, 3]_def")), 3);
