@@ -1,4 +1,4 @@
-## Copyright 2014 Oliver Heimlich
+## Copyright 2014-2015 Oliver Heimlich
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published by
@@ -33,6 +33,12 @@ if (nargin ~= 1)
     return
 endif
 
-result = strcmp (interval.dec, "ill");
+## NaI is internally stored as a scalar empty interval with ill decoration.
+## Therefore, we do not have to perform an expensive strcmp operation on large
+## matrices when it suffices to look at the number of elements.
+result = numel (interval.dec) == 1 && strcmp (interval.dec, "ill");
 
-return
\ No newline at end of file
+endfunction
+
+%!assert (isnai (infsupdec ("[nai]")));
+%!assert (not (isnai (infsupdec (2, 3))));
